@@ -53,8 +53,15 @@
 #endif
 #include <time.h>
 
-#ifdef HAVE_GSSAPI
+#ifdef HAVE_GSSAPI_H
+#include <gssapi.h>
+#endif
+
+#ifdef HAVE_GSSAPI_GSSAPI_H
 #include <gssapi/gssapi.h>
+#endif
+
+#ifdef HAVE_GSSAPI_GSSAPI_GENERIC_H
 #include <gssapi/gssapi_generic.h>
 #endif
 
@@ -201,7 +208,7 @@ FTP_DEBUG (FtpConnection *conn,
 
 #else 
 
-#define ftp_debug(c,g) (g)
+#define ftp_debug(c,g)
 
 #endif
 
@@ -967,7 +974,7 @@ ftp_kerberos_login (FtpConnection *conn,
 
 	send_tok.value = g_strdup_printf ("host@%s", saved_ip);
 	send_tok.length = strlen (send_tok.value) + 1;
-	maj_stat = gss_import_name (&min_stat, &send_tok, gss_nt_service_name,
+	maj_stat = gss_import_name (&min_stat, &send_tok, GSS_C_NT_HOSTBASED_SERVICE,
 				    &target_name);
 	g_free (send_tok.value);
 	if (maj_stat != GSS_S_COMPLETE) {
@@ -1125,12 +1132,16 @@ static GnomeVFSResult
 try_kerberos (GnomeVFSURI *uri,
 	      char **saved_ip,
 	      FtpConnection *conn,
-	      const char *user, 
+	      const char *user,
+	      gboolean *connection_failed,
 	      GnomeVFSCancellation *cancellation)
 {
-#ifdef HAVE_GSSAPI
 	GnomeVFSResult result;
 
+	/* Even without kerberos we do this login so we
+	   can see if the connection fails. The connection will
+	   be reused if we succeed */
+	*connection_failed = FALSE;
 	if (conn->socket_buf == NULL) {
 		result = try_connection (uri,
 					 saved_ip,
@@ -1138,10 +1149,12 @@ try_kerberos (GnomeVFSURI *uri,
 					 cancellation);
 		
 		if (result != GNOME_VFS_OK) {
+			*connection_failed = TRUE;
 			return result;
 		}
 	}
 	
+#ifdef HAVE_GSSAPI
         result = ftp_kerberos_login (conn, user, *saved_ip, cancellation);
 	
         if (result != GNOME_VFS_OK) {
@@ -1193,6 +1206,25 @@ try_login (GnomeVFSURI *uri,
 	return result;	
 }
 
+static void toggle_winnt_into_unix_mode(FtpConnection *conn, 
+					GnomeVFSCancellation *cancellation)
+{
+	do_basic_command (conn, "SITE DIRSTYLE", cancellation);
+
+	if (conn->response_message != NULL
+	 && strstr(conn->response_message, "is on") != NULL)
+	{
+		/*
+		 * msdos directory mode was already off, but
+		 * we just toggled it on.  toggle it back off again
+		 */
+
+		do_basic_command(conn, "SITE DIRSTYLE", cancellation);
+	}
+
+	/* server should now be in unix directory style */
+}
+
 static GnomeVFSResult 
 ftp_connection_create (FtpConnectionPool *pool,
 		       FtpConnection **connptr,
@@ -1209,6 +1241,7 @@ ftp_connection_create (FtpConnectionPool *pool,
 	gboolean uri_has_username;
 	gboolean got_connection;
 	gboolean ret;
+	gboolean connection_failed;
 	
 	cancellation = get_cancellation (context);
 	
@@ -1224,8 +1257,16 @@ ftp_connection_create (FtpConnectionPool *pool,
 
 	result = try_kerberos (uri, &pool->ip, conn,
 			       gnome_vfs_uri_get_user_name (uri),
+			       &connection_failed,
 			       cancellation);
-	if (result == GNOME_VFS_OK) {
+	if (connection_failed) {
+		gnome_vfs_uri_unref (conn->uri);
+		g_string_free (conn->response_buffer, TRUE);
+		g_free (conn);
+		g_free (user);
+		g_free (pass);
+		return result;
+	} else if (result == GNOME_VFS_OK) {
 		/* Logged in successfully using kerberos */
 	} else if (pool->user != NULL &&
 		   pool->password != NULL) {
@@ -1350,7 +1391,15 @@ ftp_connection_create (FtpConnectionPool *pool,
 	if (pool->server_type == NULL) {
 		do_basic_command (conn, "SYST", cancellation);
 		pool->server_type = g_strdup (conn->response_message);
+
 	}
+
+	/* if this is a windows server, toggle it into NT mode */
+
+	if (strncmp (pool->server_type, "Windows_NT", 10) == 0) {
+		toggle_winnt_into_unix_mode(conn, cancellation);
+	}
+
 	conn->server_type = g_strdup (pool->server_type);
 
 	*connptr = conn;
@@ -1386,18 +1435,18 @@ ftp_connection_destroy (FtpConnection *conn,
 	        gnome_vfs_socket_buffer_destroy (conn->data_socketbuf, TRUE, cancellation);
 
 #ifdef HAVE_GSSAPI
-	if (conn->gcontext != NULL) {
+	if (conn->gcontext != GSS_C_NO_CONTEXT) {
 		gss_qop_t maj_stat, min_stat;
 		gss_buffer_desc output_tok;
 		
 		maj_stat = gss_delete_sec_context (&min_stat,
-						   conn->gcontext,
+						   &conn->gcontext,
 						   &output_tok);
 		
 		if (maj_stat == GSS_S_COMPLETE) {
 			gss_release_buffer (&min_stat, &output_tok);
 		}
-		conn->gcontext = NULL;
+		conn->gcontext = GSS_C_NO_CONTEXT;
 	}
 #endif
 	
@@ -1884,97 +1933,6 @@ do_tell (GnomeVFSMethod *method,
  * only one line from that string. Fill in the appropriate fields of file_info.
  * return TRUE if a directory entry was found, FALSE otherwise
  */
-static gboolean 
-winnt_ls_to_file_info (gchar *ls, GnomeVFSFileInfo *file_info, 
-		       GnomeVFSFileInfoOptions options) 
-{
-	char *mtime_str;
-	int m, d, y, h, mn;
-	const char *mime_type;
-
-	/* check parameters */
-	g_return_val_if_fail (file_info != NULL, FALSE);
-
-	/* fill in bits of valid_fields as we go along */
-	file_info->valid_fields = 0;
-
-	/* First 17 chars are DOS date */
-	file_info->mtime = 0;
-	mtime_str = g_strndup (ls, 17);
-	if (sscanf (mtime_str, "%2d-%2d-%2d  %2d:%2d",
-	                &m, &d, &y, &h, &mn) == 5) {
-		/* yes it's a dos date */
-		struct tm mtime_parts;
-		mtime_parts.tm_mon = m - 1;   /* tm_mon is zero-based */
-		mtime_parts.tm_mday = d;
-		mtime_parts.tm_year = y >= 70 ? y : y + 100;  /* handle y2k */
-		mtime_parts.tm_hour = strcasecmp (mtime_str + 15, "pm") == 0 ?
-                                        h + 12 : h;
-		mtime_parts.tm_min = mn;
-		mtime_parts.tm_sec = 0;
-		mtime_parts.tm_isdst = -1;
-		file_info->mtime = mktime (&mtime_parts);
-		file_info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_MTIME;
-	}
-	/* TODO: if there isn't a date it's probably a Unix-style ftp server
-	 * running under Windows */
-
-	g_free (mtime_str);
-
-	/* just in case client doesn't check valid_fields */
-	file_info->atime = file_info->mtime;
-	file_info->ctime = file_info->mtime;
-
-	/* filename begins in column 39 */
-	if (strlen (ls) >= 39) {
-		int i;
-		i = strcspn (ls + 39, "\r\n");
-		file_info->name = g_strndup (ls + 39, i);
-	} else {
-		file_info->name = NULL;
-		return FALSE;
-	}
-
-	/* if it's a directory, columns 24-29 contains "<DIR>" */
-	file_info->type = GNOME_VFS_FILE_TYPE_REGULAR;
-	if (strlen (ls) >= 24) {
-		char *dirflag_str;
-		dirflag_str = g_strndup (ls + 24, 5);
-		if (strcmp (dirflag_str, "<DIR>") == 0) {
-			file_info->type = GNOME_VFS_FILE_TYPE_DIRECTORY;
-		}
-		g_free (dirflag_str);
-	}
-	file_info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_TYPE;
-
-	/* if not a directory, we should find right-aligned size ending
-	 * at column 37 */
-	if (file_info->type == GNOME_VFS_FILE_TYPE_REGULAR &&
-	                strlen (ls) > 17) {
-		file_info->size = strtol (ls + 17, NULL, 0);
-		file_info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_SIZE;
-	}
-
-	/* mime type */
-	if (file_info->type == GNOME_VFS_FILE_TYPE_REGULAR) {
-		mime_type = gnome_vfs_mime_type_from_name_or_default (
-		                 file_info->name,
-		                 GNOME_VFS_MIME_TYPE_UNKNOWN);
-	} else {
-		mime_type = gnome_vfs_mime_type_from_mode (S_IFDIR);
-	}
-	file_info->mime_type = g_strdup (mime_type);
-	file_info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE;
-
-	/* fill in other structures with meaningful data, even though
-	 * it may not be valid */
-	file_info->permissions = GNOME_VFS_PERM_USER_ALL |
-	                         GNOME_VFS_PERM_GROUP_ALL |
-	                         GNOME_VFS_PERM_OTHER_ALL;
-	file_info->flags = GNOME_VFS_FILE_FLAGS_NONE;
-
-	return TRUE;
-}
 
 /**
  * return TRUE if entry found, FALSE otherwise
@@ -2487,11 +2445,7 @@ do_read_directory (GnomeVFSMethod *method,
 	while (TRUE) {
 		gboolean success;
                 
-		if (strncmp (handle->server_type, "Windows_NT", 10) == 0) {
-			success = winnt_ls_to_file_info (handle->dirlistptr, file_info,
-			                                 handle->file_info_options);
-		}
-		else if (strncmp (handle->server_type, "NETWARE", 7) == 0) {
+		if (strncmp (handle->server_type, "NETWARE", 7) == 0) {
 			success = netware_ls_to_file_info (handle->dirlistptr, file_info,
 			                                   handle->file_info_options);
 		}
